<?php

namespace EventPortal\Tests\Unit\Agenda\Actions;

use EventPortal\Domain\Agenda\Actions\CreateDefaultAgenda;
use EventPortal\Domain\Agenda\Actions\UpdateSession;
use EventPortal\Domain\Agenda\Models\Agenda;
use EventPortal\Domain\Agenda\Models\Pathway;
use EventPortal\Domain\Agenda\Models\Session;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventVenue;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Support\Carbon;
use Illuminate\Validation\ValidationException;

class UpdateSessionTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function can_update_session_for_pathway()
    {
        $event = factory(Event::class)->create();
        factory(Pathway::class)->state('default')->create([
            'event_id' => $event->id,
        ]);

        (new CreateDefaultAgenda)->execute($event, '2020-05-20');

        $agenda = Agenda::first();
        $session = Session::first();

        $this->assertEquals('Example Session', $session->name);

        (new UpdateSession)->execute($session, $agenda, [
            'session-name' => 'Lunch Break',
            'session-description' => 'Example session description',
            'session-location' => 'Reception',
            'session-start-time' => '09:00',
            'session-end-time' => '10:00',
        ]);

        $this->assertEquals('Lunch Break', $session->name);
    }

    /** @test **/
    public function does_not_check_for_overlap_of_current_session()
    {
        $event = factory(Event::class)->create();
        factory(Pathway::class)->state('default')->create([
            'event_id' => $event->id,
        ]);


        (new CreateDefaultAgenda)->execute($event, '2020-05-20');

        $agenda = Agenda::first();
        $session = Session::first();

        (new UpdateSession)->execute($session, $agenda, [
            'session-name' => 'Lunch Break',
            'session-description' => 'Example session description',
            'session-location' => 'Reception',
            'session-start-time' => '09:00',
            'session-end-time' => '10:00',
        ]);

        $this->assertEquals('Lunch Break', $session->name);
    }

    /** @test **/
    public function cannot_update_a_session_that_completely_overlaps()
    {
        $event = factory(Event::class)->create();
        factory(Pathway::class)->state('default')->create([
            'event_id' => $event->id,
        ]);

        (new CreateDefaultAgenda)->execute($event, '2020-05-20');

        $sessionA = factory(Session::class)->create([
            'starting_at' => Carbon::parse('09:00:00'),
            'ending_at' => Carbon::parse('10:00:00'),
        ]);

        $sessionB = factory(Session::class)->create([
            'starting_at' => Carbon::parse('10:00:00'),
            'ending_at' => Carbon::parse('11:00:00'),

        ]);

        $agenda = Agenda::first();

        try {
            (new UpdateSession())->execute($sessionA, $agenda, [
                'session-name' => 'Lunch Break',
                'session-start-time' => '09:00',
                'session-end-time' => '11:30',
                'session-location' => 'Reception',
                'session-description' => 'Session description',
            ]);
        } catch (ValidationException $exception) {
            $this->assertArrayHasKey('session-start-time', $exception->errors());
            $this->assertArrayHasKey('session-end-time', $exception->errors());

            return;
        }

        $this->fail('Validation exception was not thrown.');
    }

    /** @test */
    public function can_add_venue()
    {
        $event = factory(Event::class)->create();
        factory(Pathway::class)->state('default')->create([
            'event_id' => $event->id,
        ]);

        $eventVenue = factory(EventVenue::class)->create();

        (new CreateDefaultAgenda)->execute($event, '2020-05-20');

        $agenda = Agenda::first();
        $session = Session::first();

        (new UpdateSession)->execute($session, $agenda, [
            'session-name' => 'Lunch Break',
            'session-description' => 'Example session description',
            'session-location' => 'Reception',
            'session-start-time' => '09:00',
            'session-end-time' => '10:00',
            'session-venue' => [
                'name' => 'Venue 1',
                'value' => $eventVenue->id,
            ],
        ]);

        $this->assertInstanceOf(EventVenue::class, $session->eventVenue);
    }
}
